//---------------------------------------------------------------------------
//
// *** formsC unit ***
// (TFormC, TFormTaskbarC, TMainFormC, TMonitorList)
//
// by Arash Ramin - http://www.digitalroom.net
// version 1.20 (August 09, 2001)
//
// (see formsC.txt for documentation)
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#define COMPILE_MULTIMON_STUBS
#include "formsC.h"
//---------------------------------------------------------------------------
TMonitorList *Monitors;
//---------------------------------------------------------------------------

/************************************************************************/
/* formsC - GLOBALS *****************************************************/
/************************************************************************/

//---------------------------------------------------------------------------
/* ensure that the form is not displayed off the screen, or in a multi-monitor
environment displayed entirely on one monitor (multimonitor code from MSDN) */
TPoint __fastcall CheckPosition(HWND hWnd)
{
  HMONITOR hMonitor;
  MONITORINFO mInfo;
  RECT rWnd, rMon;
  TPoint cPoint;

  GetWindowRect(hWnd, &rWnd);

  // find 'working area' of current monitor that window resides on
  hMonitor = MonitorFromWindow(hWnd, MONITOR_DEFAULTTONEAREST);
  mInfo.cbSize = sizeof(mInfo);
  GetMonitorInfo(hMonitor, &mInfo);
  rMon = mInfo.rcWork;

  int width = rWnd.right - rWnd.left;
  int height = rWnd.bottom - rWnd.top;

  // make sure window is visible and on 1 display
  cPoint.x = max(rMon.left, min(rMon.right - width, rWnd.left));
  cPoint.y = max(rMon.top, min(rMon.bottom - height, rWnd.top));

  SetWindowPos(hWnd, NULL, cPoint.x, cPoint.y, 0, 0,
               SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);

  return cPoint;
}
//---------------------------------------------------------------------------
/* centers the window (hWnd) relative to the parent window (hParent) */
TPoint __fastcall CenterForm(HWND hWnd, HWND hParent)
{

  RECT rWnd;    // window to be centered
  RECT rParent; // parent window

  GetWindowRect(hWnd, &rWnd);
  GetWindowRect(hParent, &rParent);

  int width = rWnd.right - rWnd.left;
  int height = rWnd.bottom - rWnd.top;

  // center relative to parent window
  rWnd.left = rParent.left + ((rParent.right - rParent.left) - width) / 2;
  rWnd.top = rParent.top + ((rParent.bottom - rParent.top) - height) / 2;

  SetWindowPos(hWnd, NULL, rWnd.left, rWnd.top, 0, 0,
               SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);

  return CheckPosition(hWnd);
}
//---------------------------------------------------------------------------
#ifdef MAINFORM_FIX
void __fastcall AppMinimize()
{
  // minimize the application with animation
  Application->NormalizeTopMosts();
  SetActiveWindow(Application->Handle);
  SetWindowPos(Application->Handle, HWND_BOTTOM, Application->MainForm->Left,
               Application->MainForm->Top, Application->MainForm->Width, 0,
               SWP_NOZORDER);
  ShowWindow(Application->Handle, SW_MINIMIZE);
  PlaySound("Minimize",NULL,SND_ALIAS | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT);

  if (Application->OnMinimize) {
    Application->OnMinimize(Application);
  }
}
#endif
//---------------------------------------------------------------------------
#ifdef MAINFORM_FIX
void __fastcall AppRestore()
{
  // restore the application with animation
  SetActiveWindow(Application->Handle);
  ShowWindow(Application->Handle, SW_RESTORE);
  SetWindowPos(Application->Handle, HWND_TOP, Screen->Width/2,
               Screen->Height/2, 0, 0, SWP_NOZORDER);
  Application->RestoreTopMosts();
  if (Screen->ActiveControl) {
    SetFocus(Screen->ActiveControl->Handle);
  }
  PlaySound("RestoreUp",NULL,SND_ALIAS | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT);

  if (Application->OnRestore) {
    Application->OnRestore(Application);
  }

}
#endif
//---------------------------------------------------------------------------

/************************************************************************/
/* TFormC - PUBLIC ******************************************************/
/************************************************************************/

//---------------------------------------------------------------------------
// sets on top property to form without the annoying flicker effect
// WARNING - this does not update the TForm:FormStyle property
void __fastcall TFormC::SetOnTop(bool OnTop)
{
  if (OnTop) {
    SetWindowPos(Handle, HWND_TOPMOST, Left, Top, Width, Height, NULL);
  }
  else {
    SetWindowPos(Handle, HWND_NOTOPMOST, Left, Top, Width, Height, NULL);
  }
}
//---------------------------------------------------------------------------
// same as TForm::Show, but ensures entire form is on 1 display
void __fastcall TFormC::Show()
{
  CheckPosition(Handle);
  inherited::Show();
}
//---------------------------------------------------------------------------
// same as TForm::ShowModal, but ensures entire form is on 1 display
int __fastcall TFormC::ShowModal()
{
  CheckPosition(Handle);
  return inherited::ShowModal();
}
//---------------------------------------------------------------------------
// same as TForm::Show, but centers relative to parent
void __fastcall TFormC::ShowCenter(HWND hParent)
{
  CenterForm(Handle, hParent);
  inherited::Show();
}
//---------------------------------------------------------------------------
// same as TForm::ShowModal, but centers relative to parent
int __fastcall TFormC::ShowModalCenter(HWND hParent)
{
  CenterForm(Handle, hParent);
  return inherited::ShowModal();
}
//---------------------------------------------------------------------------

/************************************************************************/
/* TFormTaskbarC - PROTECTED ********************************************/
/************************************************************************/

//---------------------------------------------------------------------------
// modify form Params to minimize to the taskbar
void __fastcall TFormTaskbarC::CreateParams(TCreateParams &Params)
{
  inherited::CreateParams(Params);
  Params.ExStyle |= WS_EX_APPWINDOW;
  Params.WndParent = GetDesktopWindow();
}
//---------------------------------------------------------------------------

/************************************************************************/
/* TMainFormC - PUBLIC **************************************************/
/************************************************************************/

//---------------------------------------------------------------------------
__fastcall TMainFormC::TMainFormC(TComponent* AOwner) : inherited(AOwner)
{
  // WARNING - use OnAppMessage instead of Application->OnMessage
  Application->OnMessage = AppMessageC;
}
//---------------------------------------------------------------------------
void __fastcall TMainFormC::AppMessageC(tagMSG &Msg, bool &Handled)
{
  if ((Msg.message == WM_SYSCOMMAND) && (Msg.hwnd == Application->Handle)) {
    switch (Msg.wParam) {
    #ifdef MAINFORM_FIX
    case SC_MINIMIZE:
      AppMinimize();
      Handled = true;
      break;
    case SC_RESTORE:
      AppRestore();
      Handled = true;
      break;
    #endif
    case SC_CLOSE:
      if (MinimizeOnClose) {
        #ifdef MAINFORM_FIX
        AppMinimize();
        #else
        Application->Minimize();
        #endif
        Handled = true;
      }
      break;
    }
  }

  if (OnAppMessage) {
    OnAppMessage(Msg, Handled);
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainFormC::WMSysCommand(TWMSysCommand &Message)
{
  if ((Message.CmdType == SC_CLOSE) && (MinimizeOnClose)) {
    #ifdef MAINFORM_FIX
    AppMinimize();
    #else
    Application->Minimize();
    #endif
  }
  #ifdef MAINFORM_FIX
  else if (Message.CmdType == SC_MINIMIZE) {
    AppMinimize();
  }
  else if ((Message.CmdType == SC_RESTORE) && (WindowState != wsMaximized)) {
    AppRestore();
  }
  #endif
  else {
    inherited::Dispatch(&Message);
  }
}
//---------------------------------------------------------------------------

/************************************************************************/
/* TMonitorList - GLOBAL ************************************************/
/************************************************************************/

//---------------------------------------------------------------------------
BOOL CALLBACK MonitorEnumProc(HMONITOR hMonitor, HDC hdcMonitor,
  LPRECT lprcMonitor, LPARAM dwData)
{
  TMonitorList *cInstance = (TMonitorList *)dwData;
  monitorInfo *monitor = new monitorInfo;
  monitor->Handle = hMonitor;
  cInstance->FMonitorList->Add(monitor);
  return true;
}
//---------------------------------------------------------------------------

/************************************************************************/
/* TMonitorList - PROTECTED *********************************************/
/************************************************************************/

//---------------------------------------------------------------------------
int __fastcall TMonitorList::GetCount()
{
  return FMonitorList->Count;
}
//---------------------------------------------------------------------------
monitorInfo * __fastcall TMonitorList::GetMonInfo(int Index)
{
  monitorInfo *monitor = (monitorInfo *)FMonitorList->Items[Index];

  /* retrieve monitor info on the fly since it might have changed
  since we first called EnumDisplayMonitors */
  MONITORINFO monInfo;
  monInfo.cbSize = sizeof(MONITORINFO);

  if (GetMonitorInfo(monitor->Handle, &monInfo) == 0) {
    throw Exception("Invalid monitor handle.");
  }

  monitor->ScreenArea = monInfo.rcMonitor;
  monitor->WorkArea = monInfo.rcWork;
  return monitor;
}
//---------------------------------------------------------------------------

/************************************************************************/
/* TMonitorList - PUBLIC ************************************************/
/************************************************************************/

//---------------------------------------------------------------------------
TMonitorList::TMonitorList()
{
  FMonitorList = new TList;
  EnumDisplayMonitors(NULL, NULL, MonitorEnumProc, long(this));
}
//---------------------------------------------------------------------------
TMonitorList::~TMonitorList()
{
  for (int i = 0; i < FMonitorList->Count; i++) {
    delete FMonitorList->Items[i];
  }
  delete FMonitorList;
}
//---------------------------------------------------------------------------
